@@ -30,8 +30,11 @@ import (
 	k8sv1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/client-go/tools/cache"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	"kubevirt.io/client-go/api"
+
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/kubevirt/pkg/testutils"
 	"kubevirt.io/kubevirt/pkg/virt-api/webhooks"
@@ -39,28 +42,45 @@ import (
 )
 
 var _ = Describe("Validating MigrationCreate Admitter", func() {
-	config, configMapInformer, _, _ := testutils.NewFakeClusterConfig(&k8sv1.ConfigMap{})
+	config, _, kvInformer := testutils.NewFakeClusterConfigUsingKVConfig(&v1.KubeVirtConfiguration{})
 
 	// Mock VirtualMachineInstanceMigration
 	var ctrl *gomock.Controller
 	var virtClient *kubecli.MockKubevirtClient
+	var vmiInformer cache.SharedIndexInformer
+	var migrationCreateAdmitter *MigrationCreateAdmitter
+
 	ctrl = gomock.NewController(GinkgoT())
 	migrationInterface := kubecli.NewMockVirtualMachineInstanceMigrationInterface(ctrl)
 	virtClient = kubecli.NewMockKubevirtClient(ctrl)
 	virtClient.EXPECT().VirtualMachineInstanceMigration("default").Return(migrationInterface).AnyTimes()
 
-	migrationCreateAdmitter := &MigrationCreateAdmitter{ClusterConfig: config, VirtClient: virtClient}
-
 	enableFeatureGate := func(featureGate string) {
-		testutils.UpdateFakeClusterConfig(configMapInformer, &k8sv1.ConfigMap{
-			Data: map[string]string{virtconfig.FeatureGatesKey: featureGate},
+		testutils.UpdateFakeKubeVirtClusterConfig(kvInformer, &v1.KubeVirt{
+			Spec: v1.KubeVirtSpec{
+				Configuration: v1.KubeVirtConfiguration{
+					DeveloperConfiguration: &v1.DeveloperConfiguration{
+						FeatureGates: []string{featureGate},
+					},
+				},
+			},
 		})
 	}
 	disableFeatureGates := func() {
-		testutils.UpdateFakeClusterConfig(configMapInformer, &k8sv1.ConfigMap{})
+		testutils.UpdateFakeKubeVirtClusterConfig(kvInformer, &v1.KubeVirt{
+			Spec: v1.KubeVirtSpec{
+				Configuration: v1.KubeVirtConfiguration{
+					DeveloperConfiguration: &v1.DeveloperConfiguration{
+						FeatureGates: make([]string, 0),
+					},
+				},
+			},
+		})
 	}
 
 	BeforeEach(func() {
+		vmiInformer, _ = testutils.NewFakeInformerFor(&v1.VirtualMachineInstance{})
+		migrationCreateAdmitter = &MigrationCreateAdmitter{ClusterConfig: config, VirtClient: virtClient, VMIInformer: vmiInformer}
 		migrationInterface.EXPECT().List(gomock.Any()).Return(&v1.VirtualMachineInstanceMigrationList{}, nil).Times(1)
 	})
 
@@ -97,10 +117,9 @@ var _ = Describe("Validating MigrationCreate Admitter", func() {
 	})
 
 	It("should accept valid Migration spec on create", func() {
-		vmi := v1.NewMinimalVMI("testvmimigrate1")
+		vmi := api.NewMinimalVMI("testvmimigrate1")
 
-		informers := webhooks.GetInformers()
-		informers.VMIInformer.GetIndexer().Add(vmi)
+		migrationCreateAdmitter.VMIInformer.GetIndexer().Add(vmi)
 
 		migration := v1.VirtualMachineInstanceMigration{
 			ObjectMeta: metav1.ObjectMeta{
@@ -128,10 +147,9 @@ var _ = Describe("Validating MigrationCreate Admitter", func() {
 	})
 
 	It("should reject valid Migration spec on create when feature gate isn't enabled", func() {
-		vmi := v1.NewMinimalVMI("testvmimigrate1")
+		vmi := api.NewMinimalVMI("testvmimigrate1")
 
-		informers := webhooks.GetInformers()
-		informers.VMIInformer.GetIndexer().Add(vmi)
+		migrationCreateAdmitter.VMIInformer.GetIndexer().Add(vmi)
 
 		migration := v1.VirtualMachineInstanceMigration{
 			ObjectMeta: metav1.ObjectMeta{
@@ -196,15 +214,14 @@ var _ = Describe("Validating MigrationCreate Admitter", func() {
 	})
 
 	It("should accept Migration spec on create when previous VMI migration completed", func() {
-		vmi := v1.NewMinimalVMI("testmigratevmi4")
+		vmi := api.NewMinimalVMI("testmigratevmi4")
 		vmi.Status.MigrationState = &v1.VirtualMachineInstanceMigrationState{
 			MigrationUID: "123",
 			Completed:    true,
 			Failed:       false,
 		}
 
-		informers := webhooks.GetInformers()
-		informers.VMIInformer.GetIndexer().Add(vmi)
+		migrationCreateAdmitter.VMIInformer.GetIndexer().Add(vmi)
 
 		migration := v1.VirtualMachineInstanceMigration{
 			ObjectMeta: metav1.ObjectMeta{
@@ -232,11 +249,10 @@ var _ = Describe("Validating MigrationCreate Admitter", func() {
 	})
 
 	It("should reject Migration spec on create when VMI is finalized", func() {
-		vmi := v1.NewMinimalVMI("testmigratevmi3")
+		vmi := api.NewMinimalVMI("testmigratevmi3")
 		vmi.Status.Phase = v1.Succeeded
 
-		informers := webhooks.GetInformers()
-		informers.VMIInformer.GetIndexer().Add(vmi)
+		migrationCreateAdmitter.VMIInformer.GetIndexer().Add(vmi)
 
 		migration := v1.VirtualMachineInstanceMigration{
 			ObjectMeta: metav1.ObjectMeta{
@@ -264,7 +280,7 @@ var _ = Describe("Validating MigrationCreate Admitter", func() {
 	})
 
 	It("should reject Migration spec for non-migratable VMIs", func() {
-		vmi := v1.NewMinimalVMI("testmigratevmi3")
+		vmi := api.NewMinimalVMI("testmigratevmi3")
 		vmi.Status.Phase = v1.Running
 		vmi.Status.Conditions = []v1.VirtualMachineInstanceCondition{
 			{
@@ -279,8 +295,7 @@ var _ = Describe("Validating MigrationCreate Admitter", func() {
 			},
 		}
 
-		informers := webhooks.GetInformers()
-		informers.VMIInformer.GetIndexer().Add(vmi)
+		migrationCreateAdmitter.VMIInformer.GetIndexer().Add(vmi)
 
 		migration := v1.VirtualMachineInstanceMigration{
 			ObjectMeta: metav1.ObjectMeta{
