@@ -23,13 +23,19 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"reflect"
+	"sort"
+	"strconv"
 	"strings"
 	"sync"
 	"time"
 
+	"kubevirt.io/api/migrations/v1alpha1"
+
 	k8sv1 "k8s.io/api/core/v1"
-	"k8s.io/api/policy/v1beta1"
+	policyv1 "k8s.io/api/policy/v1"
+	"k8s.io/apimachinery/pkg/api/equality"
+	"k8s.io/apimachinery/pkg/api/errors"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/types"
@@ -45,7 +51,7 @@ import (
 
 	"kubevirt.io/kubevirt/pkg/util/migrations"
 
-	virtv1 "kubevirt.io/client-go/api/v1"
+	virtv1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
 	"kubevirt.io/kubevirt/pkg/controller"
@@ -57,23 +63,45 @@ const (
 	failedToProcessDeleteNotificationErrMsg   = "Failed to process delete notification"
 	successfulUpdatePodDisruptionBudgetReason = "SuccessfulUpdate"
 	failedUpdatePodDisruptionBudgetReason     = "FailedUpdate"
+	failedGetAttractionPodsFmt                = "failed to get attachment pods: %v"
 )
 
+// This is the timeout used when a target pod is stuck in
+// a pending unschedulable state.
+const defaultUnschedulablePendingTimeoutSeconds = int64(60 * 5)
+
+// This is how many finalized migration objects left in
+// the system before we begin garbage collecting the oldest
+// migration objects
+const defaultFinalizedMigrationGarbageCollectionBuffer = 5
+
+// This is catch all timeout used when a target pod is stuck in
+// a in the pending phase for any reason. The theory behind this timeout
+// being longer than the unschedulable timeout is that we don't necessarily
+// know all the reasons a pod will be stuck in pending for an extended
+// period of time, so we want to make this timeout long enough that it doesn't
+// cause the migration to fail when it could have reasonably succeeded.
+const defaultCatchAllPendingTimeoutSeconds = int64(60 * 15)
+
 type MigrationController struct {
-	templateService    services.TemplateService
-	clientset          kubecli.KubevirtClient
-	Queue              workqueue.RateLimitingInterface
-	vmiInformer        cache.SharedIndexInformer
-	podInformer        cache.SharedIndexInformer
-	migrationInformer  cache.SharedIndexInformer
-	nodeInformer       cache.SharedIndexInformer
-	pvcInformer        cache.SharedIndexInformer
-	pdbInformer        cache.SharedIndexInformer
-	recorder           record.EventRecorder
-	podExpectations    *controller.UIDTrackingControllerExpectations
-	migrationStartLock *sync.Mutex
-	clusterConfig      *virtconfig.ClusterConfig
-	statusUpdater      *status.MigrationStatusUpdater
+	templateService         services.TemplateService
+	clientset               kubecli.KubevirtClient
+	Queue                   workqueue.RateLimitingInterface
+	vmiInformer             cache.SharedIndexInformer
+	podInformer             cache.SharedIndexInformer
+	migrationInformer       cache.SharedIndexInformer
+	nodeInformer            cache.SharedIndexInformer
+	pvcInformer             cache.SharedIndexInformer
+	pdbInformer             cache.SharedIndexInformer
+	migrationPolicyInformer cache.SharedIndexInformer
+	recorder                record.EventRecorder
+	podExpectations         *controller.UIDTrackingControllerExpectations
+	migrationStartLock      *sync.Mutex
+	clusterConfig           *virtconfig.ClusterConfig
+	statusUpdater           *status.MigrationStatusUpdater
+
+	unschedulablePendingTimeoutSeconds int64
+	catchAllPendingTimeoutSeconds      int64
 }
 
 func NewMigrationController(templateService services.TemplateService,
@@ -83,26 +111,31 @@ func NewMigrationController(templateService services.TemplateService,
 	nodeInformer cache.SharedIndexInformer,
 	pvcInformer cache.SharedIndexInformer,
 	pdbInformer cache.SharedIndexInformer,
+	migrationPolicyInformer cache.SharedIndexInformer,
 	recorder record.EventRecorder,
 	clientset kubecli.KubevirtClient,
 	clusterConfig *virtconfig.ClusterConfig,
 ) *MigrationController {
 
 	c := &MigrationController{
-		templateService:    templateService,
-		Queue:              workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-migration"),
-		vmiInformer:        vmiInformer,
-		podInformer:        podInformer,
-		migrationInformer:  migrationInformer,
-		nodeInformer:       nodeInformer,
-		pvcInformer:        pvcInformer,
-		pdbInformer:        pdbInformer,
-		recorder:           recorder,
-		clientset:          clientset,
-		podExpectations:    controller.NewUIDTrackingControllerExpectations(controller.NewControllerExpectations()),
-		migrationStartLock: &sync.Mutex{},
-		clusterConfig:      clusterConfig,
-		statusUpdater:      status.NewMigrationStatusUpdater(clientset),
+		templateService:         templateService,
+		Queue:                   workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-migration"),
+		vmiInformer:             vmiInformer,
+		podInformer:             podInformer,
+		migrationInformer:       migrationInformer,
+		nodeInformer:            nodeInformer,
+		pvcInformer:             pvcInformer,
+		pdbInformer:             pdbInformer,
+		migrationPolicyInformer: migrationPolicyInformer,
+		recorder:                recorder,
+		clientset:               clientset,
+		podExpectations:         controller.NewUIDTrackingControllerExpectations(controller.NewControllerExpectations()),
+		migrationStartLock:      &sync.Mutex{},
+		clusterConfig:           clusterConfig,
+		statusUpdater:           status.NewMigrationStatusUpdater(clientset),
+
+		unschedulablePendingTimeoutSeconds: defaultUnschedulablePendingTimeoutSeconds,
+		catchAllPendingTimeoutSeconds:      defaultCatchAllPendingTimeoutSeconds,
 	}
 
 	c.vmiInformer.AddEventHandler(cache.ResourceEventHandlerFuncs{
@@ -181,7 +214,7 @@ func ensureSelectorLabelPresent(migration *virtv1.VirtualMachineInstanceMigratio
 func (c *MigrationController) patchVMI(origVMI, newVMI *virtv1.VirtualMachineInstance) error {
 	var ops []string
 
-	if !reflect.DeepEqual(origVMI.Status.MigrationState, newVMI.Status.MigrationState) {
+	if !equality.Semantic.DeepEqual(origVMI.Status.MigrationState, newVMI.Status.MigrationState) {
 		newState, err := json.Marshal(newVMI.Status.MigrationState)
 		if err != nil {
 			return err
@@ -199,7 +232,7 @@ func (c *MigrationController) patchVMI(origVMI, newVMI *virtv1.VirtualMachineIns
 		}
 	}
 
-	if !reflect.DeepEqual(origVMI.Labels, newVMI.Labels) {
+	if !equality.Semantic.DeepEqual(origVMI.Labels, newVMI.Labels) {
 		newLabels, err := json.Marshal(newVMI.Labels)
 		if err != nil {
 			return err
@@ -213,7 +246,7 @@ func (c *MigrationController) patchVMI(origVMI, newVMI *virtv1.VirtualMachineIns
 	}
 
 	if len(ops) > 0 {
-		_, err := c.clientset.VirtualMachineInstance(origVMI.Namespace).Patch(origVMI.Name, types.JSONPatchType, controller.GeneratePatchBytes(ops))
+		_, err := c.clientset.VirtualMachineInstance(origVMI.Namespace).Patch(origVMI.Name, types.JSONPatchType, controller.GeneratePatchBytes(ops), &v1.PatchOptions{})
 		if err != nil {
 			return err
 		}
@@ -290,6 +323,13 @@ func (c *MigrationController) execute(key string) error {
 		return syncErr
 	}
 
+	if migration.IsFinal() {
+		err = c.garbageCollectFinalizedMigrations(vmi)
+		if err != nil {
+			return err
+		}
+	}
+
 	return nil
 }
 
@@ -340,7 +380,7 @@ func (c *MigrationController) updateStatus(migration *virtv1.VirtualMachineInsta
 		pod = pods[0]
 
 		if attachmentPods, err := controller.AttachmentPods(pod, c.podInformer); err != nil {
-			return fmt.Errorf("failed to get attachment pods: %v", err)
+			return fmt.Errorf(failedGetAttractionPodsFmt, err)
 		} else {
 			attachmentPodExists = len(attachmentPods) > 0
 			if attachmentPodExists {
@@ -444,7 +484,9 @@ func (c *MigrationController) updateStatus(migration *virtv1.VirtualMachineInsta
 				}
 			}
 		case virtv1.MigrationScheduled:
-			if vmi.Status.MigrationState != nil && vmi.Status.MigrationState.TargetNode != "" {
+			if vmi.Status.MigrationState != nil &&
+				vmi.Status.MigrationState.MigrationUID == migration.UID &&
+				vmi.Status.MigrationState.TargetNode != "" {
 				migrationCopy.Status.Phase = virtv1.MigrationPreparingTarget
 			}
 		case virtv1.MigrationPreparingTarget:
@@ -464,12 +506,12 @@ func (c *MigrationController) updateStatus(migration *virtv1.VirtualMachineInsta
 		}
 	}
 
-	if !reflect.DeepEqual(migration.Status, migrationCopy.Status) {
+	if !equality.Semantic.DeepEqual(migration.Status, migrationCopy.Status) {
 		err := c.statusUpdater.UpdateStatus(migrationCopy)
 		if err != nil {
 			return err
 		}
-	} else if !reflect.DeepEqual(migration.Finalizers, migrationCopy.Finalizers) {
+	} else if !equality.Semantic.DeepEqual(migration.Finalizers, migrationCopy.Finalizers) {
 		_, err := c.clientset.VirtualMachineInstanceMigration(migrationCopy.Namespace).Update(migrationCopy)
 		if err != nil {
 			return err
@@ -479,9 +521,9 @@ func (c *MigrationController) updateStatus(migration *virtv1.VirtualMachineInsta
 	return nil
 }
 
-func (c *MigrationController) createTargetPod(migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance) error {
+func (c *MigrationController) createTargetPod(migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance, sourcePod *k8sv1.Pod) error {
 
-	templatePod, err := c.templateService.RenderLaunchManifest(vmi)
+	templatePod, err := c.templateService.RenderMigrationManifest(vmi, sourcePod)
 	if err != nil {
 		return fmt.Errorf("failed to render launch manifest: %v", err)
 	}
@@ -528,6 +570,18 @@ func (c *MigrationController) createTargetPod(migration *virtv1.VirtualMachineIn
 		}
 	}
 
+	// This is used by the functional test to simulate failures
+	computeImageOverride, ok := migration.Annotations[virtv1.FuncTestMigrationTargetImageOverrideAnnotation]
+	if ok && computeImageOverride != "" {
+		for i, container := range templatePod.Spec.Containers {
+			if container.Name == "compute" {
+				container.Image = computeImageOverride
+				templatePod.Spec.Containers[i] = container
+				break
+			}
+		}
+	}
+
 	key := controller.MigrationKey(migration)
 	c.podExpectations.ExpectCreations(key, 1)
 	pod, err := c.clientset.CoreV1().Pods(vmi.GetNamespace()).Create(context.Background(), templatePod, v1.CreateOptions{})
@@ -541,7 +595,7 @@ func (c *MigrationController) createTargetPod(migration *virtv1.VirtualMachineIn
 	return nil
 }
 
-func (c *MigrationController) expandPDB(pdb *v1beta1.PodDisruptionBudget, vmi *virtv1.VirtualMachineInstance, vmim *virtv1.VirtualMachineInstanceMigration) error {
+func (c *MigrationController) expandPDB(pdb *policyv1.PodDisruptionBudget, vmi *virtv1.VirtualMachineInstance, vmim *virtv1.VirtualMachineInstanceMigration) error {
 	minAvailable := 2
 
 	if pdb.Spec.MinAvailable.IntValue() == minAvailable && pdb.Labels[virtv1.MigrationNameLabel] == vmim.Name {
@@ -551,7 +605,7 @@ func (c *MigrationController) expandPDB(pdb *v1beta1.PodDisruptionBudget, vmi *v
 
 	patch := []byte(fmt.Sprintf(`{"spec":{"minAvailable": %d},"metadata":{"labels":{"%s": "%s"}}}`, minAvailable, virtv1.MigrationNameLabel, vmim.Name))
 
-	_, err := c.clientset.PolicyV1beta1().PodDisruptionBudgets(pdb.Namespace).Patch(context.Background(), pdb.Name, types.StrategicMergePatchType, patch, v1.PatchOptions{})
+	_, err := c.clientset.PolicyV1().PodDisruptionBudgets(pdb.Namespace).Patch(context.Background(), pdb.Name, types.StrategicMergePatchType, patch, v1.PatchOptions{})
 	if err != nil {
 		c.recorder.Eventf(vmi, k8sv1.EventTypeWarning, failedUpdatePodDisruptionBudgetReason, "Error expanding the PodDisruptionBudget %s: %v", pdb.Name, err)
 		return err
@@ -608,7 +662,7 @@ func (c *MigrationController) handleTargetPodHandoff(migration *virtv1.VirtualMa
 	if controller.VMIHasHotplugVolumes(vmiCopy) {
 		attachmentPods, err := controller.AttachmentPods(pod, c.podInformer)
 		if err != nil {
-			return fmt.Errorf("failed to get attachment pods: %v", err)
+			return fmt.Errorf(failedGetAttractionPodsFmt, err)
 		}
 		if len(attachmentPods) > 0 {
 			log.Log.Object(migration).Infof("Target attachment pod for vmi %s/%s: %s", vmiCopy.Namespace, vmiCopy.Name, string(attachmentPods[0].UID))
@@ -618,7 +672,12 @@ func (c *MigrationController) handleTargetPodHandoff(migration *virtv1.VirtualMa
 		}
 	}
 
-	err := c.patchVMI(vmi, vmiCopy)
+	err := c.matchMigrationPolicy(vmiCopy, c.clusterConfig.GetMigrationConfiguration())
+	if err != nil {
+		return fmt.Errorf("failed to match migration policy: %v", err)
+	}
+
+	err = c.patchVMI(vmi, vmiCopy)
 	if err != nil {
 		c.recorder.Eventf(migration, k8sv1.EventTypeWarning, FailedHandOverPodReason, fmt.Sprintf("Failed to set MigrationStat in VMI status. :%v", err))
 		return err
@@ -633,7 +692,7 @@ func (c *MigrationController) handleSignalMigrationAbort(migration *virtv1.Virtu
 	vmiCopy := vmi.DeepCopy()
 
 	vmiCopy.Status.MigrationState.AbortRequested = true
-	if !reflect.DeepEqual(vmi.Status, vmiCopy.Status) {
+	if !equality.Semantic.DeepEqual(vmi.Status, vmiCopy.Status) {
 		newStatus, err := json.Marshal(vmiCopy.Status)
 		if err != nil {
 			return err
@@ -644,7 +703,7 @@ func (c *MigrationController) handleSignalMigrationAbort(migration *virtv1.Virtu
 		}
 		test := fmt.Sprintf(`{ "op": "test", "path": "/status", "value": %s }`, string(oldStatus))
 		patch := fmt.Sprintf(`{ "op": "replace", "path": "/status", "value": %s }`, string(newStatus))
-		_, err = c.clientset.VirtualMachineInstance(vmi.Namespace).Patch(vmi.Name, types.JSONPatchType, []byte(fmt.Sprintf("[ %s, %s ]", test, patch)))
+		_, err = c.clientset.VirtualMachineInstance(vmi.Namespace).Patch(vmi.Name, types.JSONPatchType, []byte(fmt.Sprintf("[ %s, %s ]", test, patch)), &v1.PatchOptions{})
 		if err != nil {
 			msg := fmt.Sprintf("failed to set MigrationState in VMI status. :%v", err)
 			c.recorder.Eventf(migration, k8sv1.EventTypeWarning, FailedAbortMigrationReason, msg)
@@ -657,12 +716,12 @@ func (c *MigrationController) handleSignalMigrationAbort(migration *virtv1.Virtu
 	return nil
 }
 
-func isMigrationProtected(pdb *v1beta1.PodDisruptionBudget) bool {
+func isMigrationProtected(pdb *policyv1.PodDisruptionBudget) bool {
 	return pdb.Status.DesiredHealthy == 2 && pdb.Generation == pdb.Status.ObservedGeneration
 }
 
-func filterOutOldPDBs(pdbList []*v1beta1.PodDisruptionBudget) []*v1beta1.PodDisruptionBudget {
-	var filteredPdbs []*v1beta1.PodDisruptionBudget
+func filterOutOldPDBs(pdbList []*policyv1.PodDisruptionBudget) []*policyv1.PodDisruptionBudget {
+	var filteredPdbs []*policyv1.PodDisruptionBudget
 
 	for i := range pdbList {
 		if !pdbs.IsPDBFromOldMigrationController(pdbList[i]) {
@@ -672,7 +731,7 @@ func filterOutOldPDBs(pdbList []*v1beta1.PodDisruptionBudget) []*v1beta1.PodDisr
 	return filteredPdbs
 }
 
-func (c *MigrationController) handleTargetPodCreation(key string, migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance) error {
+func (c *MigrationController) handleTargetPodCreation(key string, migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance, sourcePod *k8sv1.Pod) error {
 
 	c.migrationStartLock.Lock()
 	defer c.migrationStartLock.Unlock()
@@ -696,6 +755,7 @@ func (c *MigrationController) handleTargetPodCreation(key string, migration *vir
 
 	// XXX: Make this configurable, think about limit per node, bandwidth per migration, and so on.
 	if len(runningMigrations) >= int(*c.clusterConfig.GetMigrationConfiguration().ParallelMigrationsPerCluster) {
+		log.Log.Object(migration).Infof("Waiting to schedule target pod for vmi [%s/%s] migration because total running parallel migration count [%d] is currently at the global cluster limit.", vmi.Namespace, vmi.Name, len(runningMigrations))
 		// Let's wait until some migrations are done
 		c.Queue.AddAfter(key, time.Second*5)
 		return nil
@@ -710,6 +770,7 @@ func (c *MigrationController) handleTargetPodCreation(key string, migration *vir
 	if outboundMigrations >= int(*c.clusterConfig.GetMigrationConfiguration().ParallelOutboundMigrationsPerNode) {
 		// Let's ensure that we only have two outbound migrations per node
 		// XXX: Make this configurable, thinkg about inbound migration limit, bandwidh per migration, and so on.
+		log.Log.Object(migration).Infof("Waiting to schedule target pod for vmi [%s/%s] migration because total running parallel outbound migrations on target node [%d] has hit outbound migrations per node limit.", vmi.Namespace, vmi.Name, outboundMigrations)
 		c.Queue.AddAfter(key, time.Second*5)
 		return nil
 	}
@@ -742,7 +803,7 @@ func (c *MigrationController) handleTargetPodCreation(key string, migration *vir
 				return nil
 			}
 		}
-		return c.createTargetPod(migration, vmi)
+		return c.createTargetPod(migration, vmi, sourcePod)
 	}
 	return nil
 }
@@ -791,11 +852,133 @@ func (c *MigrationController) createAttachmentPod(migration *virtv1.VirtualMachi
 	return nil
 }
 
+func isPodPendingUnschedulable(pod *k8sv1.Pod) bool {
+
+	if pod.Status.Phase != k8sv1.PodPending || pod.DeletionTimestamp != nil {
+		return false
+	}
+
+	for _, condition := range pod.Status.Conditions {
+		if condition.Type == k8sv1.PodScheduled &&
+			condition.Status == k8sv1.ConditionFalse &&
+			condition.Reason == k8sv1.PodReasonUnschedulable {
+
+			return true
+		}
+	}
+	return false
+}
+
+func timeSinceCreationSeconds(objectMeta *metav1.ObjectMeta) int64 {
+
+	now := time.Now().UTC().Unix()
+	creationTime := objectMeta.CreationTimestamp.Time.UTC().Unix()
+	seconds := now - creationTime
+	if seconds < 0 {
+		seconds = 0
+	}
+
+	return seconds
+}
+
+func (c *MigrationController) deleteTimedOutTargetPod(migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance, pod *k8sv1.Pod, reason string) error {
+
+	migrationKey, err := controller.KeyFunc(migration)
+	if err != nil {
+		return err
+	}
+
+	c.podExpectations.ExpectDeletions(migrationKey, []string{controller.PodKey(pod)})
+	err = c.clientset.CoreV1().Pods(vmi.Namespace).Delete(context.Background(), pod.Name, v1.DeleteOptions{})
+	if err != nil {
+		c.podExpectations.DeletionObserved(migrationKey, controller.PodKey(pod))
+		c.recorder.Eventf(migration, k8sv1.EventTypeWarning, FailedDeletePodReason, "Error deleted migration target pod: %v", err)
+		return fmt.Errorf("failed to delete vmi migration target pod that reached pending pod timeout period.: %v", err)
+	}
+	log.Log.Object(vmi).Infof("Deleted pending migration target pod %s/%s with uuid %s for migration %s with uuid %s with reason [%s]", pod.Namespace, pod.Name, string(pod.UID), migration.Name, string(migration.UID), reason)
+	c.recorder.Eventf(migration, k8sv1.EventTypeNormal, SuccessfulDeletePodReason, reason, pod.Name)
+	return nil
+}
+
+func (c *MigrationController) getUnschedulablePendingTimeoutSeconds(migration *virtv1.VirtualMachineInstanceMigration) int64 {
+	timeout := c.unschedulablePendingTimeoutSeconds
+	customTimeoutStr, ok := migration.Annotations[virtv1.MigrationUnschedulablePodTimeoutSecondsAnnotation]
+	if !ok {
+		return timeout
+	}
+
+	newTimeout, err := strconv.Atoi(customTimeoutStr)
+	if err != nil {
+		log.Log.Object(migration).Reason(err).Errorf("Unable to parse unschedulable pending timeout value for migration")
+		return timeout
+	}
+
+	return int64(newTimeout)
+}
+
+func (c *MigrationController) getCatchAllPendingTimeoutSeconds(migration *virtv1.VirtualMachineInstanceMigration) int64 {
+	timeout := c.catchAllPendingTimeoutSeconds
+	customTimeoutStr, ok := migration.Annotations[virtv1.MigrationPendingPodTimeoutSecondsAnnotation]
+	if !ok {
+		return timeout
+	}
+
+	newTimeout, err := strconv.Atoi(customTimeoutStr)
+	if err != nil {
+		log.Log.Object(migration).Reason(err).Errorf("Unable to parse catch all pending timeout value for migration")
+		return timeout
+	}
+
+	return int64(newTimeout)
+}
+
+func (c *MigrationController) handlePendingPodTimeout(migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance, pod *k8sv1.Pod) error {
+
+	if pod.Status.Phase != k8sv1.PodPending || pod.DeletionTimestamp != nil || pod.CreationTimestamp.IsZero() {
+		// only check if timeout has occurred if pod is pending and not already marked for deletion
+		return nil
+	}
+
+	migrationKey, err := controller.KeyFunc(migration)
+	if err != nil {
+		return err
+	}
+
+	unschedulableTimeout := c.getUnschedulablePendingTimeoutSeconds(migration)
+	catchAllTimeout := c.getCatchAllPendingTimeoutSeconds(migration)
+	secondsSpentPending := timeSinceCreationSeconds(&pod.ObjectMeta)
+
+	if isPodPendingUnschedulable(pod) {
+		c.alertIfHostModelIsUnschedulable(vmi, pod)
+		c.recorder.Eventf(
+			migration,
+			k8sv1.EventTypeWarning,
+			MigrationTargetPodUnschedulable,
+			"Migration target pod for VMI [%s/%s] is currently unschedulable.", vmi.Namespace, vmi.Name)
+		log.Log.Object(migration).Warningf("Migration target pod for VMI [%s/%s] is currently unschedulable.", vmi.Namespace, vmi.Name)
+		if secondsSpentPending >= unschedulableTimeout {
+			return c.deleteTimedOutTargetPod(migration, vmi, pod, "unschedulable pod timeout period exceeded")
+		} else {
+			// Make sure we check this again after some time
+			c.Queue.AddAfter(migrationKey, time.Second*time.Duration(unschedulableTimeout-secondsSpentPending))
+		}
+	}
+
+	if secondsSpentPending >= catchAllTimeout {
+		return c.deleteTimedOutTargetPod(migration, vmi, pod, "pending pod timeout period exceeded")
+	} else {
+		// Make sure we check this again after some time
+		c.Queue.AddAfter(migrationKey, time.Second*time.Duration(catchAllTimeout-secondsSpentPending))
+	}
+
+	return nil
+}
+
 func (c *MigrationController) sync(key string, migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance, pods []*k8sv1.Pod) error {
 
 	var pod *k8sv1.Pod = nil
-	podExists := len(pods) > 0
-	if podExists {
+	targetPodExists := len(pods) > 0
+	if targetPodExists {
 		pod = pods[0]
 	}
 
@@ -817,28 +1000,47 @@ func (c *MigrationController) sync(key string, migration *virtv1.VirtualMachineI
 
 	switch migration.Status.Phase {
 	case virtv1.MigrationPending:
-		if !podExists {
-			return c.handleTargetPodCreation(key, migration, vmi)
+		if !targetPodExists {
+			sourcePod, err := controller.CurrentVMIPod(vmi, c.podInformer)
+			if err != nil {
+				log.Log.Reason(err).Error("Failed to fetch pods for namespace from cache.")
+				return err
+			}
+			if !podExists(sourcePod) {
+				// for instance sudden deletes can cause this. In this
+				// case we don't have to do anything in the creation flow anymore.
+				// Once the VMI is in a final state or deleted the migration
+				// will be marked as failed too.
+				return nil
+			}
+			return c.handleTargetPodCreation(key, migration, vmi, sourcePod)
 		} else if isPodReady(pod) {
 			if controller.VMIHasHotplugVolumes(vmi) {
 				attachmentPods, err := controller.AttachmentPods(pod, c.podInformer)
 				if err != nil {
-					return fmt.Errorf("failed to get attachment pods: %v", err)
+					return fmt.Errorf(failedGetAttractionPodsFmt, err)
 				}
 				if len(attachmentPods) == 0 {
 					log.Log.Object(migration).Infof("Creating attachment pod for vmi %s/%s on node %s", vmi.Namespace, vmi.Name, pod.Spec.NodeName)
 					return c.createAttachmentPod(migration, vmi, pod)
 				}
 			}
+		} else {
+			return c.handlePendingPodTimeout(migration, vmi, pod)
 		}
+	case virtv1.MigrationScheduling:
+		if targetPodExists {
+			return c.handlePendingPodTimeout(migration, vmi, pod)
+		}
+
 	case virtv1.MigrationScheduled:
 		// once target pod is running, then alert the VMI of the migration by
 		// setting the target and source nodes. This kicks off the preparation stage.
-		if podExists && isPodReady(pod) {
+		if targetPodExists && isPodReady(pod) {
 			return c.handleTargetPodHandoff(migration, vmi, pod)
 		}
 	case virtv1.MigrationPreparingTarget, virtv1.MigrationTargetReady, virtv1.MigrationFailed:
-		if (!podExists || podIsDown(pod)) &&
+		if (!targetPodExists || podIsDown(pod)) &&
 			vmi.Status.MigrationState != nil &&
 			len(vmi.Status.MigrationState.TargetDirectMigrationNodePorts) == 0 &&
 			vmi.Status.MigrationState.StartTimestamp == nil &&
@@ -981,7 +1183,7 @@ func (c *MigrationController) updatePod(old, cur interface{}) {
 		return
 	}
 
-	labelChanged := !reflect.DeepEqual(curPod.Labels, oldPod.Labels)
+	labelChanged := !equality.Semantic.DeepEqual(curPod.Labels, oldPod.Labels)
 	if curPod.DeletionTimestamp != nil {
 		// having a pod marked for deletion is enough to count as a deletion expectation
 		c.deletePod(curPod)
@@ -994,7 +1196,7 @@ func (c *MigrationController) updatePod(old, cur interface{}) {
 
 	curControllerRef := c.getControllerOf(curPod)
 	oldControllerRef := c.getControllerOf(oldPod)
-	controllerRefChanged := !reflect.DeepEqual(curControllerRef, oldControllerRef)
+	controllerRefChanged := !equality.Semantic.DeepEqual(curControllerRef, oldControllerRef)
 	if controllerRefChanged && oldControllerRef != nil {
 		// The ControllerRef was changed. Sync the old controller, if any.
 		if migration := c.resolveControllerRef(oldPod.Namespace, oldControllerRef); migration != nil {
@@ -1047,8 +1249,8 @@ func (c *MigrationController) deletePod(obj interface{}) {
 }
 
 func (c *MigrationController) updatePDB(old, cur interface{}) {
-	curPDB := cur.(*v1beta1.PodDisruptionBudget)
-	oldPDB := old.(*v1beta1.PodDisruptionBudget)
+	curPDB := cur.(*policyv1.PodDisruptionBudget)
+	oldPDB := old.(*policyv1.PodDisruptionBudget)
 	if curPDB.ResourceVersion == oldPDB.ResourceVersion {
 		return
 	}
@@ -1074,6 +1276,62 @@ func (c *MigrationController) updatePDB(old, cur interface{}) {
 	}
 }
 
+type vmimCollection []*virtv1.VirtualMachineInstanceMigration
+
+func (c vmimCollection) Len() int {
+	return len(c)
+}
+
+func (c vmimCollection) Less(i, j int) bool {
+	t1 := &c[i].CreationTimestamp
+	t2 := &c[j].CreationTimestamp
+	return t1.Before(t2)
+}
+
+func (c vmimCollection) Swap(i, j int) {
+	c[i], c[j] = c[j], c[i]
+}
+
+func (c *MigrationController) garbageCollectFinalizedMigrations(vmi *virtv1.VirtualMachineInstance) error {
+
+	var finalizedMigrations []string
+
+	migrations, err := c.listMigrationsMatchingVMI(vmi.Namespace, vmi.Name)
+	if err != nil {
+		return err
+	}
+
+	// Oldest first
+	sort.Sort(vmimCollection(migrations))
+	for _, migration := range migrations {
+		if migration.IsFinal() && migration.DeletionTimestamp == nil {
+			finalizedMigrations = append(finalizedMigrations, migration.Name)
+		}
+	}
+
+	// only keep the oldest 5 finalized migration objects
+	garbageCollectionCount := len(finalizedMigrations) - defaultFinalizedMigrationGarbageCollectionBuffer
+
+	if garbageCollectionCount <= 0 {
+		return nil
+	}
+
+	for i := 0; i < garbageCollectionCount; i++ {
+		err = c.clientset.VirtualMachineInstanceMigration(vmi.Namespace).Delete(finalizedMigrations[i], &v1.DeleteOptions{})
+		if err != nil && errors.IsNotFound(err) {
+			// This is safe to ignore. It's possible in some
+			// scenarios that the migration we're trying to garbage
+			// collect has already disappeared. Let's log it as debug
+			// and suppress the error in this situation.
+			log.Log.V(3).Reason(err).Infof("error encountered when garbage collecting migration object %s/%s", vmi.Namespace, finalizedMigrations[i])
+		} else if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
 // takes a namespace and returns all migrations listening for this vmi
 func (c *MigrationController) listMigrationsMatchingVMI(namespace string, name string) ([]*virtv1.VirtualMachineInstanceMigration, error) {
 	objs, err := c.migrationInformer.GetIndexer().ByIndex(cache.NamespaceIndex, namespace)
@@ -1116,7 +1374,7 @@ func (c *MigrationController) updateVMI(old, cur interface{}) {
 		// have different RVs.
 		return
 	}
-	labelChanged := !reflect.DeepEqual(curVMI.Labels, oldVMI.Labels)
+	labelChanged := !equality.Semantic.DeepEqual(curVMI.Labels, oldVMI.Labels)
 	if curVMI.DeletionTimestamp != nil {
 		// having a DataVOlume marked for deletion is enough
 		// to count as a deletion expectation
@@ -1223,14 +1481,49 @@ func (c *MigrationController) getNodeForVMI(vmi *virtv1.VirtualMachineInstance)
 	return node, nil
 }
 
+func (c *MigrationController) alertIfHostModelIsUnschedulable(vmi *virtv1.VirtualMachineInstance, targetPod *k8sv1.Pod) {
+	fittingNodeFound := false
+
+	if cpu := vmi.Spec.Domain.CPU; cpu == nil || cpu.Model != virtv1.CPUModeHostModel {
+		return
+	}
+
+	requiredNodeLabels := map[string]string{}
+	for key, value := range targetPod.Spec.NodeSelector {
+		if strings.HasPrefix(key, virtv1.HostModelCPULabel) || strings.HasPrefix(key, virtv1.CPUFeatureLabel) {
+			requiredNodeLabels[key] = value
+		}
+	}
+
+	nodes := c.nodeInformer.GetStore().List()
+	for _, nodeInterface := range nodes {
+		node := nodeInterface.(*k8sv1.Node)
+
+		if node.Name == vmi.Status.NodeName {
+			continue // avoid checking the VMI's source node
+		}
+
+		if isNodeSuitableForHostModelMigration(node, requiredNodeLabels) {
+			log.Log.Object(vmi).Infof("Node %s is suitable to run vmi %s host model cpu mode (more nodes may fit as well)", node.Name, vmi.Name)
+			fittingNodeFound = true
+			break
+		}
+	}
+
+	if !fittingNodeFound {
+		warningMsg := fmt.Sprintf("Migration cannot proceed since no node is suitable to run the required CPU model / required features: %v", requiredNodeLabels)
+		c.recorder.Eventf(vmi, k8sv1.EventTypeWarning, NoSuitableNodesForHostModelMigration, warningMsg)
+		log.Log.Object(vmi).Warning(warningMsg)
+	}
+}
+
 func prepareNodeSelectorForHostCpuModel(node *k8sv1.Node, pod *k8sv1.Pod) error {
-	var hostCpuModel string
-	var labelValue string
+	var hostCpuModel, hostCpuModelLabelKey, hostModelLabelValue string
 
 	for key, value := range node.Labels {
 		if strings.HasPrefix(key, virtv1.HostModelCPULabel) {
 			hostCpuModel = strings.TrimPrefix(key, virtv1.HostModelCPULabel)
-			labelValue = value
+			hostModelLabelValue = value
 		}
 
 		if strings.HasPrefix(key, virtv1.HostModelRequiredFeaturesLabel) {
@@ -1243,9 +1536,60 @@ func prepareNodeSelectorForHostCpuModel(node *k8sv1.Node, pod *k8sv1.Pod) error
 		return fmt.Errorf("node does not contain labal \"%s\" with information about host cpu model", virtv1.HostModelCPULabel)
 	}
 
-	labelKey := virtv1.HostModelCPULabel + hostCpuModel
-	pod.Spec.NodeSelector[labelKey] = labelValue
-	log.Log.Object(pod).Infof("host model label selector (\"%s\") defined for migration target pod", labelKey)
+	hostCpuModelLabelKey = virtv1.HostModelCPULabel + hostCpuModel
+	pod.Spec.NodeSelector[hostCpuModelLabelKey] = hostModelLabelValue
+
+	log.Log.Object(pod).Infof("host model label selector (\"%s\") defined for migration target pod", hostCpuModelLabelKey)
+
+	return nil
+}
+
+func isNodeSuitableForHostModelMigration(node *k8sv1.Node, requiredNodeLabels map[string]string) bool {
+	for key, value := range requiredNodeLabels {
+		nodeValue, ok := node.Labels[key]
+
+		if !ok || nodeValue != value {
+			return false
+		}
+	}
+
+	return true
+}
+
+func (c *MigrationController) matchMigrationPolicy(vmi *virtv1.VirtualMachineInstance, clusterMigrationConfiguration *virtv1.MigrationConfiguration) error {
+	vmiNamespace, err := c.clientset.CoreV1().Namespaces().Get(context.Background(), vmi.Namespace, v1.GetOptions{})
+	if err != nil {
+		return err
+	}
+
+	// Fetch cluster policies
+	var policies []v1alpha1.MigrationPolicy
+	migrationInterfaceList := c.migrationPolicyInformer.GetStore().List()
+	for _, obj := range migrationInterfaceList {
+		policy := obj.(*v1alpha1.MigrationPolicy)
+		policies = append(policies, *policy)
+	}
+	policiesListObj := v1alpha1.MigrationPolicyList{Items: policies}
+
+	// Override cluster-wide migration configuration if migration policy is matched
+	matchedPolicy := policiesListObj.MatchPolicy(vmi, vmiNamespace)
+
+	if matchedPolicy == nil {
+		log.Log.Object(vmi).Reason(err).Infof("no migration policy matched for VMI %s", vmi.Name)
+		return nil
+	}
+
+	migrationConfigCopy := *clusterMigrationConfiguration
+	isUpdated, err := matchedPolicy.GetMigrationConfByPolicy(&migrationConfigCopy)
+	if err != nil {
+		return err
+	}
+
+	if isUpdated {
+		vmi.Status.MigrationState.MigrationPolicyName = &matchedPolicy.Name
+		vmi.Status.MigrationState.MigrationConfiguration = &migrationConfigCopy
+		log.Log.Object(vmi).Infof("migration is updated by migration policy named %s.", matchedPolicy.Name)
+	}
 
 	return nil
 }
