@@ -21,11 +21,11 @@ package watch
 
 import (
 	"fmt"
-	"reflect"
 	"sync"
 	"time"
 
 	k8score "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/equality"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/util/wait"
@@ -35,12 +35,14 @@ import (
 
 	"kubevirt.io/kubevirt/pkg/util/status"
 
-	virtv1 "kubevirt.io/client-go/api/v1"
+	virtv1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
 	"kubevirt.io/kubevirt/pkg/controller"
 )
 
+const failedRsKeyExtraction = "Failed to extract rsKey from replicaset."
+
 // Reasons for replicaset events
 const (
 	// FailedCreateVirtualMachineReason is added in an event and in a replica set condition
@@ -206,7 +208,7 @@ func (c *VMIReplicaSet) execute(key string) error {
 		return fresh, nil
 	})
 	cm := controller.NewVirtualMachineControllerRefManager(controller.RealVirtualMachineControl{Clientset: c.clientset}, rs, selector, virtv1.VirtualMachineInstanceReplicaSetGroupVersionKind, canAdoptFunc)
-	vmis, err = cm.ClaimVirtualMachines(vmis)
+	vmis, err = cm.ClaimVirtualMachineInstances(vmis)
 	if err != nil {
 		return err
 	}
@@ -242,7 +244,7 @@ func (c *VMIReplicaSet) scale(rs *virtv1.VirtualMachineInstanceReplicaSet, vmis
 
 	rsKey, err := controller.KeyFunc(rs)
 	if err != nil {
-		log.Log.Object(rs).Reason(err).Error("Failed to extract rsKey from replicaset.")
+		log.Log.Object(rs).Reason(err).Error(failedRsKeyExtraction)
 		return nil
 	}
 
@@ -466,7 +468,7 @@ func (c *VMIReplicaSet) updateVirtualMachine(old, cur interface{}) {
 		return
 	}
 
-	labelChanged := !reflect.DeepEqual(curVMI.Labels, oldVMI.Labels)
+	labelChanged := !equality.Semantic.DeepEqual(curVMI.Labels, oldVMI.Labels)
 	if curVMI.DeletionTimestamp != nil {
 		// when a vmi is deleted gracefully it's deletion timestamp is first modified to reflect a grace period,
 		// and after such time has passed, the virt-handler actually deletes it from the store. We receive an update
@@ -483,7 +485,7 @@ func (c *VMIReplicaSet) updateVirtualMachine(old, cur interface{}) {
 
 	curControllerRef := metav1.GetControllerOf(curVMI)
 	oldControllerRef := metav1.GetControllerOf(oldVMI)
-	controllerRefChanged := !reflect.DeepEqual(curControllerRef, oldControllerRef)
+	controllerRefChanged := !equality.Semantic.DeepEqual(curControllerRef, oldControllerRef)
 	if controllerRefChanged && oldControllerRef != nil {
 		// The ControllerRef was changed. Sync the old controller, if any.
 		if rs := c.resolveControllerRef(oldVMI.Namespace, oldControllerRef); rs != nil {
@@ -574,7 +576,7 @@ func (c *VMIReplicaSet) enqueueReplicaSet(obj interface{}) {
 	rs := obj.(*virtv1.VirtualMachineInstanceReplicaSet)
 	key, err := controller.KeyFunc(rs)
 	if err != nil {
-		logger.Object(rs).Reason(err).Error("Failed to extract rsKey from replicaset.")
+		logger.Object(rs).Reason(err).Error(failedRsKeyExtraction)
 	}
 	c.Queue.Add(key)
 }
@@ -782,7 +784,7 @@ func (c *VMIReplicaSet) resolveControllerRef(namespace string, controllerRef *me
 func (c *VMIReplicaSet) cleanFinishedVmis(rs *virtv1.VirtualMachineInstanceReplicaSet, vmis []*virtv1.VirtualMachineInstance) error {
 	rsKey, err := controller.KeyFunc(rs)
 	if err != nil {
-		log.Log.Object(rs).Reason(err).Error("Failed to extract rsKey from replicaset.")
+		log.Log.Object(rs).Reason(err).Error(failedRsKeyExtraction)
 		return nil
 	}
 
