@@ -31,7 +31,7 @@ import (
 
 func GetPodCPUSet() ([]int, error) {
 	var cpuset string
-	file, err := os.Open(cgroup.CPUSetPath())
+	file, err := os.Open(cgroup.CPUSetPath(""))
 	if err != nil {
 		return nil, err
 	}
@@ -43,7 +43,7 @@ func GetPodCPUSet() ([]int, error) {
 	if err := scanner.Err(); err != nil {
 		return nil, err
 	}
-	cpusList, err := hardware.ParseCPUSetLine(cpuset)
+	cpusList, err := hardware.ParseCPUSetLine(cpuset, 50000)
 	if err != nil {
 		return nil, fmt.Errorf("failed to parse cpuset file: %v", err)
 	}
